/*****************************************************************************
 ***  Copyright (c) Wandfluh AG  -  CH-3714 Frutigen  -  www.wandfluh.com  ***
 *****************************************************************************
 *
 * Projekt  GESO
 *
 * @author  AT
 * @date    26.11.2012
 * @file    WAGSD7_Device.ddl
 *
 * Device Description (DDL) for the Wandfluh AG - SD7 Enhanced Controller
 * with HART interface and its variants.
 *
 ****************************************************************************/

#define WAG_MANUFACTURER    254
#define WAG_DEVICE_TYPE     130

MANUFACTURER WAG_MANUFACTURER, DEVICE_TYPE WAG_DEVICE_TYPE, DEVICE_REVISION 0, DD_REVISION 1

#include "macros.h"
#include "methods.h"

#define WAG_MANUFACTURER_CODES \
    { WAG_MANUFACTURER,  "Wandfluh AG" }

#define WAG_DEVICE_CODES \
    { 130,  "Wandfluh AG - SD7 Enhanced Controller" }, \
    { 131,  "Wandfluh AG - SD7 Basic Controller" }

#define WAG_ALL_UNITS \
		ALL_UNITS, \
        UNITS_CODE(241), \
        UNITS_CODE(242), \
        UNITS_CODE(243)

/************************************************************************
*
* Common Tables
*
*************************************************************************/
IMPORT STANDARD _TABLES, DEVICE_REVISION 7, DD_REVISION 1
{
	EVERYTHING;
	
	REDEFINITIONS
	{
		VARIABLE units_code
		{
			TYPE ENUMERATED
			{
				// Add vendor specific units
				ADD { 241, [kilo_newton],   [kilo_newton_help] }
				ADD { 242, [ccm],           [cubic_centimeter_help] }
				ADD { 243, [mm_s],          [millimeters_per_second_help] }
			}
		}
	}
}

/************************************************************************
*
* Universal Commands
*
*************************************************************************/
IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 5, DD_REVISION 1
{
	EVERYTHING;

	REDEFINITIONS
	{		
		VARIABLE device_type
		{
			REDEFINE TYPE ENUMERATED
			{
				WAG_DEVICE_CODES
			}
		}
		
		VARIABLE manufacturer_id
		{
			REDEFINE TYPE ENUMERATED
			{				
				WAG_MANUFACTURER_CODES
			}
		}
		
		VARIABLE private_label_distributor
		{
			REDEFINE TYPE ENUMERATED
			{
				WAG_MANUFACTURER_CODES
			}
		}

		COMMAND read_pv
		{
			REDEFINE TRANSACTION
			{
				REQUEST
				{}

				REPLY
				{
					response_code, device_status,
					PV.DIGITAL_UNITS <0xFF>, PV.DIGITAL_VALUE,
				}
			}
		}

		COMMAND read_pv_current_and_percent_range
		{
			REDEFINE TRANSACTION
			{
				REQUEST
				{}

				REPLY
				{
					response_code, device_status,
					PV.ANALOG_VALUE, PV.PERCENT_RANGE
				}
			}
		}

		COMMAND read_dynamic_variables_and_pv_current
		{
			REDEFINE TRANSACTION
			{
				REQUEST
				{}

				REPLY
				{
					response_code, device_status,
					PV.ANALOG_VALUE,
					PV.DIGITAL_UNITS <0xFF>, PV.DIGITAL_VALUE,
					SV.DIGITAL_UNITS <0xFF>, SV.DIGITAL_VALUE,
					TV.DIGITAL_UNITS <0xFF>, TV.DIGITAL_VALUE,
					QV.DIGITAL_UNITS <0xFF>, QV.DIGITAL_VALUE
				}
			}
		}

		COMMAND read_pv_sensor_info
		{
			REDEFINE TRANSACTION
			{
				REQUEST
				{}

				REPLY
				{
					response_code, device_status,
					PV.SENSOR_SERIAL_NUMBER, PV.SENSOR_UNITS <0xFF>,
					PV.UPPER_SENSOR_LIMIT, PV.LOWER_SENSOR_LIMIT,
					PV.MINIMUM_SPAN
				}
			}
		}
		
		COMMAND read_pv_output_info
		{
			REDEFINE TRANSACTION
			{
				REQUEST
				{}

				REPLY
				{
					response_code, device_status,
					PV.ALARM_CODE, PV.TRANSFER_FUNCTION,
					PV.RANGE_UNITS <0xFF>, PV.UPPER_RANGE_VALUE, PV.LOWER_RANGE_VALUE,
					PV.DAMPING_VALUE, write_protect, private_label_distributor
				}
			}
		}
	}
}

/*****************************************************************************
 *
 * Import device specific variables
 *
 *****************************************************************************/
#define WAG_HART_PARAM_READ_LIST \
        tag, descriptor, date, message, \
        manufacturer_id, device_type, device_id, \
        universal_revision, transmitter_revision, \
        software_revision, hardware_revision, \
        device_flags, physical_signaling_code, \
        final_assembly_number

#define WAG_HART_PARAM_WRITE_LIST \
        tag, descriptor, date, message, \
        final_assembly_number

#include "WAGSD76_Params.dd"

/******************************************************************************
 * Device Variable 0: Actual Value
 *****************************************************************************/
VARIABLE var_actval_unit
{
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}

VARIABLE var_actval_value
{
	LABEL "Actual value|de|Istwert";
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

/*
 * Analog output
 */

VARIABLE var_analog_output_unit
{
	LABEL [sensor_unit];
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}

VARIABLE var_analog_output_value
{
	LABEL [loop_current];
    HANDLING READ;
    CLASS ANALOG_OUTPUT & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_analog_output_percent_range
{
	LABEL [percent_of_range];
    HANDLING READ;
    CLASS ANALOG_OUTPUT & DYNAMIC;
	CONSTANT_UNIT [Percent];
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_actval_alarm_code
{
	LABEL [alarm_select];
	CLASS DEVICE;	
	HANDLING READ;
	TYPE ENUMERATED
	{
		ALARM_SELECTION_CODE(0),
		ALARM_SELECTION_CODE(1),
	}
}

VARIABLE var_actval_transfer_function
{
	LABEL [transfer_function];
	HANDLING READ;
	CLASS DEVICE;		
	TYPE ENUMERATED
	{
		TRANSFER_FUNCTION_CODE (0),		/* Linear */
	}
}

/*
 * Analog input of PV
 */

VARIABLE var_actval_range_unit
{
	LABEL [sensor_unit];
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}


VARIABLE var_actval_upper_range
{
	LABEL [unknown];
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_actval_lower_range
{
	LABEL [sensor_unit];
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}

VARIABLE var_analog_output_upper_limit
{
	LABEL [usl];
    HANDLING READ;
    CLASS ANALOG_OUTPUT & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_analog_output_lower_limit
{
	LABEL [lsl];
    HANDLING READ;
    CLASS ANALOG_OUTPUT & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_analog_output_minimum_span
{
	LABEL [minimum_span];
    HANDLING READ;
    CLASS ANALOG_OUTPUT & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".5f";
    }
}

VARIABLE var_analog_output_serial_number
{
	LABEL [sensor_serial_number];
    HANDLING READ;
    CLASS ANALOG_OUTPUT;
    TYPE UNSIGNED_INTEGER (3);
}


VARIABLE var_actval_damping_value
{
	LABEL [damping_value];
    HANDLING READ;
    CLASS ANALOG_INPUT & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

COLLECTION OF VARIABLE col_actual
{
	LABEL [pv];
    MEMBERS
    {
        DIGITAL_UNITS,  		var_actval_unit;
        DIGITAL_VALUE,  		var_actval_value;
		
		ALARM_CODE,				var_actval_alarm_code;
		DAMPING_VALUE,			var_actval_damping_value;
		TRANSFER_FUNCTION,		var_actval_transfer_function;
		RANGE_UNITS,			var_actval_range_unit;
		UPPER_RANGE_VALUE,		var_actval_upper_range;		
		LOWER_RANGE_VALUE,		var_actval_lower_range;
		
		/* Analog Output Information */		
		SENSOR_UNITS,			var_analog_output_unit;
		ANALOG_VALUE,			var_analog_output_value;
		UPPER_SENSOR_LIMIT,		var_analog_output_upper_limit;
		LOWER_SENSOR_LIMIT,		var_analog_output_lower_limit;
		MINIMUM_SPAN,			var_analog_output_minimum_span;
		PERCENT_RANGE,			var_analog_output_percent_range;
		
		SENSOR_SERIAL_NUMBER,	var_analog_output_serial_number;
    }
}

/******************************************************************************
 * Device Variable 1: Command Input
 *****************************************************************************/
VARIABLE var_command_input_value
{
	LABEL "Analog command value|de|Analoger Sollwert";
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_command_input_unit
{
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}

COLLECTION OF VARIABLE col_command_input
{
	LABEL [sv];
    MEMBERS
    {
        DIGITAL_UNITS,  var_command_input_unit;
        DIGITAL_VALUE,  var_command_input_value;        
    }
}

/******************************************************************************
 * Device Variable 2: Command Value
 *****************************************************************************/
VARIABLE var_command_value
{
	LABEL "Command value|de|Sollwert";
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_command_unit
{
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}

COLLECTION OF VARIABLE col_command
{
	LABEL [tv];
    MEMBERS
    {
        DIGITAL_UNITS,  var_command_unit;
        DIGITAL_VALUE,  var_command_value;        
    }
}

/******************************************************************************
 * Device Variable 3: Control Deviation
 *****************************************************************************/
VARIABLE var_deviation_value
{
	LABEL "Control deviation|de|Regelabweichung";
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE FLOAT
    {
        DISPLAY_FORMAT ".3f";
    }
}

VARIABLE var_deviation_unit
{
    HANDLING READ;
    CLASS DEVICE & DYNAMIC;
    TYPE ENUMERATED (1)
    {
        WAG_ALL_UNITS
    }
}

COLLECTION OF VARIABLE col_deviation
{
	LABEL [qv];
    MEMBERS
    {
        DIGITAL_UNITS,  var_deviation_unit;
        DIGITAL_VALUE,  var_deviation_value;
    }
}

/******************************************************************************
 * Entry - point for the process variables
 *****************************************************************************/
ARRAY OF COLLECTION dynamic_variables
{
	LABEL "Dynamic variables|de|Dynamische Variablen";
	ELEMENTS
	{
		PRIMARY,	col_actual,			[pv];
		SECONDARY,	col_command_input,	[sv];
		TERTIARY,	col_command,		[tv];
		QUATERNARY,	col_deviation,		[qv];
	}
}

/******************************************************************************
 * Relations for the process variables
 *****************************************************************************/
UNIT unt_pv
{
    PV.DIGITAL_UNITS:
        PV.DIGITAL_VALUE
}

UNIT unt_analog_output
{
	PV.SENSOR_UNITS:
		PV.ANALOG_VALUE
		PV.UPPER_SENSOR_LIMIT,
		PV.LOWER_SENSOR_LIMIT,
		PV.MINIMUM_SPAN
}

UNIT unt_sv
{
    SV.DIGITAL_UNITS:
        SV.DIGITAL_VALUE
}

UNIT unt_tv
{
    TV.DIGITAL_UNITS:
        TV.DIGITAL_VALUE
}

UNIT unt_qv
{
    QV.DIGITAL_UNITS:
        QV.DIGITAL_VALUE
}

/*****************************************************************************
 *
 * Menues
 *
 *****************************************************************************/

MENU offline_root_menu
{
    LABEL 	[main_menu];
    ITEMS
    {
		Tab_ident,
		Tab_Device_Params
    }
}

MENU process_variables_root_menu
{
    LABEL "Process variables|de|Prozessgrssen";
    STYLE WINDOW;
    ITEMS
    {
        PV.DIGITAL_VALUE (DISPLAY_VALUE),
        PV.PERCENT_RANGE (DISPLAY_VALUE),
        PV.ANALOG_VALUE  (DISPLAY_VALUE),
        SEPARATOR,
        SV.DIGITAL_VALUE (DISPLAY_VALUE),
        TV.DIGITAL_VALUE (DISPLAY_VALUE),
        QV.DIGITAL_VALUE (DISPLAY_VALUE)
    }
}

MENU Menu_Main_Maintenance
{
    LABEL   [main_menu];
    ITEMS
    {
        Tab_ident
    }
}

MENU Menu_Main_Specialist
{
    LABEL 	[main_menu];
    ITEMS
    {
		Tab_ident,
		Tab_Device_Params
    }
}

MENU Tab_ident
{
    LABEL   [menu_ident];
    ITEMS
    {
        Tab_ident_opunit,
        Tab_ident_device
    }
}

MENU Tab_ident_opunit
{
    LABEL   [menu_ident_opunit];
    ITEMS
    {
        tag,
        descriptor,
        date,
        message,
        final_assembly_number
    }
}

MENU Tab_ident_device
{
    LABEL   [menu_ident_device];
    ITEMS
    {
        manufacturer_id,
        device_type,
        device_id,
        universal_revision,
        transmitter_revision,
        software_revision,
        hardware_revision,
        device_flags,
        physical_signaling_code
    }
}

